//////////////////////////////////////////////////
//#ServerEngines Public License
//#This license governs use of the accompanying software. If you use the software, you
//#accept this license. If you do not accept the license, do not use the software.
//#
//#1. Definitions
//#The terms "reproduce," "reproduction," "derivative works," and "distribution" have the
//#same meaning here as under U.S. copyright law.
//#A "contribution" is the original software, or any additions or changes to the software.
//#A "contributor" is any person that distributes its contribution under this license.
//#"Licensed patents" are a contributor's patent claims that read directly on its contribution.
//#
//##2. Grant of Rights
//###(A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//##(B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//##
//##3. Conditions and Limitations
//##(A) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//##(B) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//#(C) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//#(D) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//#(E) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.
//##
//###       Project  : Pilot
/// CONFIDENTIAL
/// Copyright (c) 2009 Server Engines
/// *All rights reserved
/// Filename: 
/// Section: U-Boot
/// Author: Dhanrajj






#include  "pilot_types.h"

#define VIC 0x40000000

//mac defines
#define MAC1BASE	0x40500000	//0x40400000
#define MAC2BASE	0x40600000	//0x40460000

#define CSR0BMR			0x0
#define CSR5SR			0x28
#define CSR6OMR			0x30

#define CSR0SWR			(0x1<<0)  		//;S/W reset
#define CSR6SR			(0x1<<1)		//;start/stop receive command
#define CSR6ST			(0x1<<13)		//;start/stop transmit command

//ssp specific
#define CC_CTL_INV_ALL            (1<<3)  
#define CC_CTL_TAG_I_D_SEL        (1<<5)
#define CC_CTL_INV_ALL            (1<<3)  

//usb2-specific
#define USB2BASE                0x40800000
#define  USB2_INTSTS_WRITE     (USB2BASE+0x1B0)
#define USB2_INTENABLE         (USB2BASE+0x1B1)
#define Dma_Control_Reg        (USB2BASE+0x1B8)
#define DMA_SETSTOP            (1<<24)  
#define  dma_inrpt           (1<<1)

//sdhc
#define SDHC_BASE              0x40C00000
#define PRSNTST_REG            0x024


#define READ_REG8(a)      (*((tPVU8)  (a)))
#define READ_REG16(a)     (*((tPVU16) (a)))
#define READ_REG32(a)     (*((tPVU32) (a)))
                                                                                                                                                                        
#define WRITE_REG8(a,d)   (*((tPVU8) (a)) = d)
#define WRITE_REG16(a,d)  (*((tPVU16) (a)) = d)
#define WRITE_REG32(a,d)  (*((tPVU32) (a)) = d)
                                                                                                                                                                        
#define SET_BIT8(a,b)     (WRITE_REG8 (a, (READ_REG8(a)  | (tU8) (1<<b))))
#define SET_BIT16(a,b)    (WRITE_REG16(a, (READ_REG16(a) | (tU16)(1<<b))))
#define SET_BIT32(a,b)    (WRITE_REG32(a, (READ_REG32(a) | (tU32) (1<<b))))
#define SET_BIT(a,b)      (SET_BIT32(a,b))
                                                                                                                                                                        
#define CLEAR_BIT8(a,b)   (WRITE_REG8 (a, (READ_REG8(a) & (tU8)(~(1<<b)))))
#define CLEAR_BIT16(a,b)  (WRITE_REG16(a, (READ_REG16(a) & (tU16)(~(1<<b)))))
#define CLEAR_BIT32(a,b)  (WRITE_REG32(a, (READ_REG32(a) & (tU32)(~(1<<b)))))
#define CLEAR_BIT(a,b)    (CLEAR_BIT32(a,b))
                                                                                                                                                                        
#define TOGGLE_BIT8(a,b)  (WRITE_REG8 (a, (READ_REG8(a)  ^ (tU8) (1<<b))))
#define TOGGLE_BIT16(a,b) (WRITE_REG16(a, (READ_REG16(a) ^ (tU16)(1<<b))))
#define TOGGLE_BIT32(a,b) (WRITE_REG32(a, (READ_REG32(a) ^ (tU32)(1<<b))))
#define TOGGLE_BIT(a,b)   (TOGGLE_BIT32(a,b))



void stop_mac_tx_rx(unsigned int mac_base)
{
  *(tPVU32)(mac_base + CSR6OMR)&=~( (CSR6SR)|(CSR6ST));
  
}

//Disable MAC transmit and Receive port_num = 0 for MAC0 and 1 for MAC1
int Wait_4_Tx_Rx_Stopped(unsigned int mac_base)
{
  //  *(tPVU32)(mac_base + CSR6OMR)&=~( (CSR6SR)|(CSR6ST));
  tU32 temp=0;
  tU32 wait_to=0;

  //Wait for TX to goto stopped state
  temp = 0xffffffff;
  wait_to = 0xffff;
  while(temp)
    {
      temp = *(tPVU32)(mac_base + CSR5SR);
      temp = (temp >> 20)&0x7;
      wait_to--;
      if(wait_to==0x0)
	{
	  wait_to=0xffff;
	  
	  break;
	}
    }
  
  //Wait for RX to goto stopped state
  temp = 0xffffffff;
  wait_to = 0xffff;
  while(temp)
    {
      temp = *(tPVU32)(mac_base + CSR5SR);
      temp = (temp >> 17)&0x7;
      wait_to--;
      if(wait_to==0x0)
	{
	  wait_to=0xffff;
	  //        nc_printf("wait for mac1  rx to be in IDLE state\n");
	  break;
	}
      
      *(tPVU32)(mac_base+CSR0BMR)=CSR0SWR;
      while(( *(tPVU32)(mac_base+CSR0BMR)) &CSR0SWR)
	{
	  //	   nc_printf("waiting for %x s/w reset bit to be cleared\n",mac_base);
	}
      
    }
  return 0;
  
}




#define FGBBASE 0x40900000

void Put_Ssp_In_Reset(void)
{
  // reset the orp processor
  *(unsigned int *)(FGBBASE)|=1;
}


void Icache_Invalidate_All(void)
{
  *(unsigned int *)(FGBBASE+0x100c)|=(CC_CTL_INV_ALL|CC_CTL_TAG_I_D_SEL);
  while( *(unsigned int *)(FGBBASE+0x100c) & (CC_CTL_INV_ALL) );
  
}


void Dcache_Invalidate_All(void)
{
  *(unsigned int *)(FGBBASE+0x100c)&=(~(CC_CTL_TAG_I_D_SEL));
  *(unsigned int *)(FGBBASE+0x100c)|=(CC_CTL_INV_ALL);
  while(*(unsigned int *)(FGBBASE+0x100c) &(CC_CTL_INV_ALL) );
}


//Disable TFE, BSE & SSP
void fgb_disable(void)
{
  *(tPVU32)(FGBBASE+0x100)=0x0; //TFE
  *(tPVU32)(FGBBASE+0x200)=0x0; //BSE
 
  
  Put_Ssp_In_Reset();
  Icache_Invalidate_All(); //SSP
  Dcache_Invalidate_All(); //SSP

}


void usb1_dma_disable(void)
{
  tU32 time_out=0xfff;
  while( !(*(tPVU8)(0x40701d1c)&(1<<5)) )
   {
	time_out--;
	if(time_out==0x0)
	break;
     //     nc_printf("usb1 dma is busy\n");
   }
 
}

void clear_usb2_dma_sts(void)
{
 *(tPVU8)(USB2_INTSTS_WRITE)=dma_inrpt;
  //clear dma interrupt
  *(tPVU8)(USB2_INTENABLE) &=(~dma_inrpt);

}


void usb2_dma_disable(void)
{
  *(tPVU8)(USB2_INTSTS_WRITE)=dma_inrpt;
  *(tPVU32)(Dma_Control_Reg) |= (DMA_SETSTOP);

}



//check if SDHC is busy
void sdhc_not_busy(void) {
  tU32 data;
  data = READ_REG32(SDHC_BASE+0x110);
  while((data & 0xf) != 0)
    data = READ_REG32(SDHC_BASE+0x110);
  data = READ_REG32(SDHC_BASE+0x118);
  while((data & 0xff000000) != 0)
    data = READ_REG32(SDHC_BASE+0x118);
  data = READ_REG32(SDHC_BASE+0x11C);
  while((data & 0xf0000000) != 0)
    data = READ_REG32(SDHC_BASE+0x11C);
  data = READ_REG32(SDHC_BASE+0x120);
  while((data & 0xff000000) != 0)
    data = READ_REG32(SDHC_BASE+0x120);
  data = READ_REG32(SDHC_BASE+0x124);
  while((data & 0xf0000000) != 0)
    data = READ_REG32(SDHC_BASE+0x124);
  data = *(tPVU32)(SDHC_BASE + PRSNTST_REG);
  while ((data & 0x00f00000) != 0x00f00000)
    data = *(tPVU32)(SDHC_BASE + PRSNTST_REG);
  //  DELAY(1000);
}

#define Bomber 0x40417000
void disable_bomber(void) {
  WRITE_REG32(Bomber, 0x00);
}



#define FGB_SECBASE 0x40A00000
void gdma_disable(void)
{
  while(*(tPVU32)(FGB_SECBASE+0x084)&0x1)
    {
      //   nc_printf("Gdma is busy \n");
      ;
    }
  *(tPVU32)(FGB_SECBASE+0x080)=0x0;
}


#define NAND_BASE 0x40D00000
//Checks if nand is busy.
void nand_not_busy(void) 
{
  tU32 data;
  tU32 time_out=0xffff;
  data = READ_REG32(NAND_BASE+0x18);
  while((data & 0x700) != 00) 
    {
      data = READ_REG32(NAND_BASE+0x18);
      //  nc_printf("NAND BUSY\n");
      time_out--;
      if(time_out==0x0)
	break;
    }
}


void ldma_disable(void)
{
  *(tPVU32)(0x40900300)&=(0xFFFFFFFE);
}


void do_soft_reset(tU8 rstno, tU8 all) {
  if(all == 1) {
    WRITE_REG32(0x4010080C, 0xffdfff3c); //excluding ARM, DMA AHB & BOOT SPI
  }
  else
    SET_BIT32(0x4010080C, rstno);
  DELAY(300);
  SET_BIT32(0x40100800, 1); //Start soft reset to modules selected above
  
  while((READ_REG32(0x40100804) & 0x02) != 0x02);
  DELAY(45000); //Wait for reset to be deasserted
  if(all == 1) {
    WRITE_REG32(0x4010080C, 0x0);
  }
  else
    CLEAR_BIT32(0x4010080C, rstno);
}

void sw_reset_modules(void)
{
  *(unsigned int *)(VIC)=0x0;
  stop_mac_tx_rx(MAC1BASE);
  stop_mac_tx_rx(MAC2BASE);
  sdhc_not_busy();
  usb2_dma_disable();
  
  
  fgb_disable();
  usb1_dma_disable();
  
  disable_bomber();
  gdma_disable();
  
  
  nand_not_busy();
  fgb_disable();
  ldma_disable();
  clear_usb2_dma_sts();
  Wait_4_Tx_Rx_Stopped(MAC1BASE);
  Wait_4_Tx_Rx_Stopped(MAC2BASE);
  
  do_soft_reset(1, 1);
  return;  
  
}

