/*
 * Copyright (C) 2007 American Megatrends Inc
 *
 * Modified by ServerEngines for Pilot3
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "spiflash.h"
#ifdef __UBOOT__	
#include "soc_hw.h"
#endif

#define SPI_INPUT_CLOCK 100		/* Input Frequency MHZ */
#define MAX_CLOCK	25		/* Max SPI Frequency MHZ supported by Controller */
#define MAX_READ	4

// Pilot-II supports multiple SPI busses.  Bank 0 is the Boot SPI, Bank 1 is the Backup/Data SPI
// Route any access to bank 1 to the backup SPI code
extern int pilot3spi_bkup_init(void);
extern int pilot3spi_bkup_transfer(int bank,unsigned char *cmd,int cmdlen, SPI_DIR dir,
                                            unsigned char *data, unsigned long datalen);

typedef struct 
{
        volatile unsigned char Addr0;           /* LSB */
        volatile unsigned char Addr1;
        volatile unsigned char Addr2;           /* MSB */
        volatile unsigned char Rsvd1;
	volatile unsigned long pilot3_Dummy;
        volatile union {
                volatile struct {
                        volatile unsigned char Opcode;
                        volatile unsigned char CmdType;
                        volatile unsigned char Dummy;
                        volatile unsigned char CmdCtrl;
                };
                volatile unsigned long Command;
        };
        volatile unsigned char Trec;
        volatile unsigned char Tcs;
        volatile unsigned char Tcsl;
        volatile unsigned char Tcsh;
        volatile unsigned char SpiCtrl;
        volatile unsigned char Rsvd2[3];
        volatile unsigned char ClkDivL;
        volatile unsigned char ClkDivH;
        volatile unsigned char Rsvd3[2];
        volatile unsigned long SpiStatus;
        volatile unsigned long Rsvd4;
	volatile unsigned char Rsvd5[16];//Pilot3 needs so much of offset as the data register is at offset 0x30
        volatile union {
                volatile unsigned char Data8[4];
                volatile unsigned long Data32;
        } Data;
} BOOT_SPI_REGS;


volatile BOOT_SPI_REGS *Regs = (volatile BOOT_SPI_REGS *)(SE_BOOT_SPI_VA_BASE);
static void inline
WaitForSpiReady(void)
{
	while (Regs->SpiStatus & 0x01);
	return;
}

static void inline
WaitForWriteFifo(void)
{
	/* Wait till Write Fifo Ready Bit is set */
	while (!(Regs->SpiStatus & 0x08));
	return;
}

static void inline
WaitForWriteFifoEmpty(void)
{
	/* Wait till Fifo Empty Bit is set */
	while (!(Regs->SpiStatus & 0x20));
	return;
}

static int inline
IsWriteFifoFull(void)
{
	return (Regs->SpiStatus & 0x10);
}

static void
SpiEnable(int en)
{
	if (en) 
		Regs->SpiCtrl |= 0x01;
	else
		Regs->SpiCtrl &= 0xFE;
	return;
}

static
void
InitializeCtrl(void)
{
	/* Disable the ctrl */
	SpiEnable(0);
	
	/* Set the clock to 25MHZ */ //We might have to fix it for ASIC
//	Regs->ClkDivH = 0;
	//Regs->ClkDivL = (SPI_INPUT_CLOCK/MAX_CLOCK);

	/* Enable the ctrl */
	SpiEnable(1);

	/* Wait for the SPI internal state to be ready */
	WaitForSpiReady();

	return;
}
	


static
int
ValidateInputs(unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, int datalen)
{
	if ((cmdlen != 1) && (cmdlen != 4) && (cmdlen !=5))
	{
		printk("pilot2spi : Invalid Command Len %d\n",cmdlen);
		return -1;
	}
	if (cmd == NULL)
	{
		printk("pilot2spi : Invalid : Null Command\n");
		return -1;
	}
	if (dir != SPI_NONE)
	{
		if ( (data == NULL) || (datalen == 0))
		{
			printk("pilot2spi : Invalid data or datalen \n");
			return -1;
		}
	}
	if ((dir == SPI_READ) && (datalen > MAX_READ))
	{
		printk("pilot2spi : Invalid : Reads cannot exceed %d bytes\n",MAX_READ);
		return -1;
	}
	
	return 0;

}

static
void
DoSpiFifoWrite(unsigned char *data, int datalen)
{

	/* Wait for Fifo Mode Enabled */
	//WaitForWriteFifo();			

	/* Transfer all data to Fifo */
	while (datalen --)
	{
		Regs->Data.Data8[0] = *data;
		data++;
		/* Wait for Fifo not full */
		//while (IsWriteFifoFull());
	}

	/* Tell ctrl data is over by deselect CS */

	/* Wait for the Fifo to be transferrred */
	//WaitForWriteFifoEmpty();		
	return;
}

#if 0
#define SHOWREG8(x) printk("Register %s @ 0x%08lx = 0x%02x\n",#x,(unsigned long)&(Regs->x),Regs->x)
#define SHOWREG32(x) printk("Register %s @ 0x%08lx = 0x%08lx\n",#x,(unsigned long)&(Regs->x),Regs->x)
static
void
DumpRegisters(void)
{
	SHOWREG8(Addr0);
	SHOWREG8(Addr1);
	SHOWREG8(Addr2);
	SHOWREG32(Command);
	SHOWREG8(ClkDivH);
	SHOWREG8(ClkDivL);
	SHOWREG32(SpiStatus);
	SHOWREG32(Data.Data32);
	return;
}
#endif

#ifdef __UBOOT__	
typedef struct 
{
	ulong length;
	ulong rsvd;
	ulong src;
	ulong dest;
} GDMA_DESCRIPTOR;

typedef struct 
{
	volatile ulong GDMA_CTRL;
	volatile ulong GDMA_STAT;
	volatile ulong GDMA_DESC;
	volatile ulong GDMA_CHK;
} GDMA_REGS;

volatile GDMA_REGS *gdma_regs = (volatile GDMA_REGS *)(SE_SECURITY_VA_BASE + 0x80);

/*Align at 16 Byte Boundary */
GDMA_DESCRIPTOR gdma_desc __attribute__ ((aligned (16)));



static
int  
pilot2spi_split_burst_read(int bank, unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, unsigned long datalen)
{
	dma_addr_t dmabuff;
	unsigned long src;
	//unsigned long timeout =100000;
	unsigned long timeout =800000;


	src = cmd[1] << 16 | cmd[2] << 8 | cmd[3];
	
	/* Convert Buffer to DMA Buffer */
	dmabuff = dma_map_single(NULL,data,datalen,DMA_FROM_DEVICE);
	if (dmabuff == 0)
	{
		printk("ERROR:pilot2spi dma_map_single() failed\n");
		return -1;
	}

	/* Fill the Descriptors */
	gdma_desc.dest = dmabuff;
	gdma_desc.src  = src;
	gdma_desc.rsvd = 0;
	gdma_desc.length = (datalen-1) << 8;

	/* Enable GDMA */
	gdma_regs->GDMA_CTRL |= 0x1;

	/* Load the Descriptor to GDMA . This starts transfer */
	gdma_regs->GDMA_DESC = (ulong)virt_to_phys(&gdma_desc);

	/* Wait for completion */	
	while ((gdma_regs->GDMA_STAT & 0x1) && timeout)
		timeout --;

	/* Disable GDMA */
	gdma_regs->GDMA_CTRL &= ~(0x1);

	
	/* Release DMA Memory */
	dma_unmap_single(NULL,dmabuff,datalen,DMA_FROM_DEVICE);

	if (timeout == 0)
	{
		printk("ERROR: pilot2spi_burst_read timeout for read at 0x%08lx for %ld bytes\n",
						src,datalen);
		return -1;
	}
	return 0;
}


#define MAX_BURST_READ 0x1000
static
int  
pilot2spi_burst_read(int bank,unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, unsigned long datalen)
{
	unsigned long transfer;
	int retval;
	unsigned long src;

    // Pass any access to bank 1 to the Backup/Data SPI handler
    if (bank == 1)
        return pilot3spi_bkup_transfer(bank, cmd, cmdlen, dir, data, datalen);

	while (datalen)	
	{
		if (datalen > MAX_BURST_READ)
			transfer = MAX_BURST_READ;
		else
			transfer = datalen;
		
		retval = pilot2spi_split_burst_read(bank,cmd,cmdlen,dir,data,transfer);
		if (retval != 0)
			return retval;
 
		/* Increment Source address */
		src = cmd[1] << 16 | cmd[2] << 8 | cmd[3];
		src+=transfer;
		cmd[1] = src >> 16;
		cmd[2] = src >> 8;
		cmd[3] = src;
		/* Increment buffer */
		data+=transfer;

		/* Decrement remain len */
		datalen-=transfer;
	}
	return 0;
}
#endif // __UBOOT__	

static
int  
pilot2spi_transfer(int bank,unsigned char *cmd,int cmdlen, SPI_DIR dir, 
				unsigned char *data, unsigned long datalen)
{
	unsigned char Opcode;
	unsigned char Type;
	unsigned char Dummy;
	unsigned char Ctrl;
	unsigned long Command;
	int i;
    // Pass any access to bank 1 to the Backup/Data SPI handler
    if (bank == 1)
        return pilot3spi_bkup_transfer(bank, cmd, cmdlen, dir, data, datalen);

	if (ValidateInputs(cmd,cmdlen,dir,data,datalen) == -1)
		return -1;

	Ctrl = (0x80|Regs->CmdCtrl);
	Dummy = 0;

	/* Fill in Command And Address */
	Opcode = cmd[0];

	
	Regs->Rsvd1 = 0x0;

	if (cmdlen >= 4)
	{
		Regs->Addr0 = cmd[3];
		Regs->Addr1 = cmd[2];	
		Regs->Addr2 = cmd[1];
	}
	

	/* If Fast Read, Fill in Dummy and Enable Fast Read Bit */
	if (cmdlen == 5)
	{
		Dummy = cmd[4];
		Ctrl |= 0x04;
	}

	
	/* Fill in Command Len and data len */
	Type = 0;
	if (dir == SPI_READ)
	{
	  Type = 0x0;
	}
	else
	{
		Type |= 0x10;
	}
	

	Type |= cmdlen;

	/* Write Command . Enable Write Fifo */
	if (dir == SPI_WRITE)
	{
		Type |= 0x10;
	}

	Command = (Opcode) | (Type << 8) | (Dummy << 16) | (Ctrl << 24);
	/* Issue Command */
	WaitForSpiReady();
	Regs->Command = Command;
	
	WaitForSpiReady();
	/* Do FIFO write for write commands */
	if (dir == SPI_WRITE)
	  DoSpiFifoWrite(data,datalen);
	
	/* Wait for SPI Controller become Idle */
	WaitForSpiReady();
	
	  {
	    /* Read the data from the data register */
	    if (dir == SPI_READ)
	      {
		for(i=0;i<datalen;i++)
		  data[i] = Regs->Data.Data8[0];
	      }
	  }


	/* Switch back to non-register mode and disable Write Fifo mode */
	Regs->CmdCtrl &= 0x7F;


	return 0;
}

#ifdef __UBOOT__	
extern void ll_serial_init(void);
static int 
pilot2spi_init(void)
{
	InitializeCtrl();	
	pilot3spi_bkup_init();
	return 0;
}
#endif

#ifndef __UBOOT__	/* linux */
struct spi_ctrl_driver pilot2spi_driver =
#else
/* 
   Uboot supports only one spi controller and has to be defined 
   in the structure "SPI_CTRL_DRIVER"
*/
struct spi_ctrl_driver SPI_CTRL_DRIVER =
#endif
{
	.name 		= "pilot2spi",
	.module 	= THIS_MODULE,
	.max_read	= MAX_READ,
	.fast_read	= 1,
	.spi_transfer	= pilot2spi_transfer,
#ifdef __UBOOT__	
	.spi_burst_read	= pilot2spi_burst_read,
	.spi_init	= pilot2spi_init,
#else
    .spi_burst_read = NULL,
#endif
};

#ifndef __UBOOT__	/* linux */
static int 
pilot2spi_init(void)
{
	InitializeCtrl();	
	init_MUTEX(&pilot2spi_driver.lock);
	register_spi_ctrl_driver(&pilot2spi_driver);
	pilot3spi_bkup_init();
	return 0;
}


static void 
pilot2spi_exit(void)
{
	unregister_spi_ctrl_driver(&pilot2spi_driver);
	return;
}

module_init(pilot2spi_init);
module_exit(pilot2spi_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("American Megatrends Inc");
MODULE_DESCRIPTION("PILOT-II SOC SPI Controller driver");

#endif
