#pragma once

#include <pilot3_hw.h>
#include "pilot3/hw_pwm.h"

#define PIN_INPUT       1
#define PIN_OUTPUT      0

// GPIO Ports
// Only GPIO ports 0 and 1 are available to the host.
// The remainder are SGPIO.
#define GPIO_PORT0      0
#define GPIO_PORT1      1

// GPIO Configuration Register bits
#define GPCFG_EVENT_TO_SMI  (1 << 7) // 1 == enabled
#define GPCFG_EVENT_TO_IRQ  (1 << 6) // 1 == enabled
#define GPCFG_DEBOUNCE_EN   (1 << 5) // 1 == input debounce, 0 == pulse output
#define GPCFG_ACTIVE_HIGH   (1 << 4) // 1 == Active high
#define GPCFG_LEVEL_TRIG    (1 << 3) // 1 == level (default), 0 == edge
#define GPCFG_PULLDOWN      (0x0 << 1) // Internal Pull-down enable
#define GPCFG_PULLUP        (0x2 << 1) // Internal Pull-up enable
#define GPCFG_PULL_DISABLE  (0x3 << 1) // No Pull-up/Pull-down
#define GPCFG_OUTPUT_EN     (1 << 0) // 1 == Output enabled, 0 == tri-state (default)

// GPIO Debounce and Blink Configuration Register bits
#define GPIO_DEBOUNCE_8ms       0x00
#define GPIO_DEBOUNCE_16ms      0x01
#define GPIO_DEBOUNCE_60us      0x02
#define GPIO_DEBOUNCE_120us     0x03

#define GPIO_BLINK_OFF          0x00
#define GPIO_BLINK_1S           0x01
#define GPIO_BLINK_2S           0x02
#define GPIO_BLINK_4S           0x03

#define PWM_PORT0       0
#define PWM_PORT1       1

#define PWM_MIN_DUTY    0
#define PWM_MAX_DUTY    1

typedef struct _GPIOValue
{
    uint32 u32GPIOPort_Pin;
    uint8  u8PinCFG;
    uint8  u8Value;
    uint8  u8Debounce;
} GPIOValue;




//
// Pointer to the Fan Control structure.
// Pin it to the PilotII memory-mapped hardware address of register space.
//
extern  HWREG_PILOT_II_FAN_CONTROL volatile  *HWREG_Pilot_II_Fan_Control;

//
// Safe defaults data for initializing PWMs in uBoot. 
//
typedef struct _PWMValue
{
  char*   Name;
  uint32  PWMFrequency; // in Hz
  uint8   PWMDutyCycle; // in %
} PWMValue;

extern PWMValue  const g_PWMValues[];

