/*
 *  This file contains the SE PILOT-II Register maps and other data
 *
 *  Copyright (C) 2007 Intel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#pragma once

#include "intel_types.h"


/* -----------------------------------------------------------------*
 *               PWM and Fan Tach Controller Registers
 * -----------------------------------------------------------------*/
#pragma pack (4)
typedef struct HWREG_PILOT_II_FAN_CONTROL_TAG {

  union {
    struct {
      uint8  PWM_Control_0_Enable : 1;
      uint8  PWM_Control_0_Invert : 1;
      uint8  PWM_Control_1_Enable : 1;
      uint8  PWM_Control_1_Invert : 1;
      uint8  PWM_Control_2_Enable : 1;
      uint8  PWM_Control_2_Invert : 1;
      uint8  PWM_Control_3_Enable : 1;
      uint8  PWM_Control_3_Invert : 1;
    }      bit;
    uint8  reg;
  } PWCFG_reg;  // PWM Control Configuration Register
  //uint8  dummy0[3];

  union {
    struct {
      uint8  Fan_Tach_Monitor_0_Enable : 1;
      uint8  Fan_Tach_Monitor_1_Enable : 1;
      uint8  Fan_Tach_Monitor_2_Enable : 1;
      uint8  Fan_Tach_Monitor_3_Enable : 1;
      uint8  Fan_Tach_Monitor_4_Enable : 1;
      uint8  Fan_Tach_Monitor_5_Enable : 1;
      uint8  Fan_Tach_Monitor_6_Enable : 1;
      uint8  Fan_Tach_Monitor_7_Enable : 1;
    }      bit;
    uint8  reg;
  } FMCFG_reg;  // Fan Tach Monitor Configuration Register
  //uint8  dummy1[3];

  union {
    struct {
      uint8  Fan_0_Enable_on_OTS             : 1;
      uint8  Fan_1_Enable_on_OTS             : 1;
      uint8  Fan_2_Enable_on_OTS             : 1;
      uint8  Fan_3_Enable_on_OTS             : 1;
      uint8  Enable_PWM_0_divide_by_128_or64 : 1;
      uint8  Enable_PWM_1_divide_by_128_or64 : 1;
      uint8  Enable_PWM_2_divide_by_128_or64 : 1;
      uint8  Enable_PWM_3_divide_by_128_or64 : 1;
    }      bit;
    uint8  reg;
  } PWOFC_reg;  // PWM Control OTS Configuration Register
  //uint8  dummy2[3];

  union {
    struct {
      uint8  Fan_Tach_0_Interrupt : 1;
      uint8  Fan_Tach_1_Interrupt : 1;
      uint8  Fan_Tach_2_Interrupt : 1;
      uint8  Fan_Tach_3_Interrupt : 1;
      uint8  Fan_Tach_4_Interrupt : 1;
      uint8  Fan_Tach_5_Interrupt : 1;
      uint8  Fan_Tach_6_Interrupt : 1;
      uint8  Fan_Tach_7_Interrupt : 1;
    }      bit;
    uint8  reg;
  } FTISR_reg;  // Fan Tach Status Interrupt Register
  //uint8  dummy3[3];

  union {
    struct {
      uint8  PWM_0_Programmable_Counter_Resolution_Enable : 1;
      uint8  PWM_0_Programmable_Counter_Resolution_Error  : 1;
      uint8  PWM_1_Programmable_Counter_Resolution_Enable : 1;
      uint8  PWM_1_Programmable_Counter_Resolution_Error  : 1;
      uint8  PWM_2_Programmable_Counter_Resolution_Enable : 1;
      uint8  PWM_2_Programmable_Counter_Resolution_Error  : 1;
      uint8  PWM_3_Programmable_Counter_Resolution_Enable : 1;
      uint8  PWM_3_Programmable_Counter_Resolution_Error  : 1;
    }      bit;
    uint8  reg;
  } PWCRCSR_reg;  // PWM Counter Resolution Control / Status Register
  //uint8  dummy4[3];

  struct {
    uint8  Value;
    //uint8  dummy4[3];
  } PWCRVR_reg[4];  // PWM Counter Resolution Value Register

  uint8  dummy5[0x40 - 0x24]; // Reserved

  struct {
    union {
      struct {
        uint8  Pre_Scale : 7;
        uint8  Clock_Set : 1;
      }      bit;
      uint8  reg;
    } PWPSR_reg;  // Pre-Scale
    //uint8 dummy0[3];
    union {
      uint8  Duty_Cycle;
    } PWDCR_reg;  // Duty Cycle
    //uint8 dummy1[3];
  } PWM_Control_reg[4]; // PWM Control

  struct {
    union {
      uint8  Threshold_Value;
    } FMTHR_reg;  // Threshold
    //uint8 dummy0[3];
    union {
      uint8  Fan_Speed_Reading;
    } FMSPR_reg;  // Speed
    //uint8 dummy1[3];
    union {
      struct {
        uint8  Speed_Ready      : 1;
        uint8  Over_Threshold   : 1;
        uint8  Overflow         : 1;
        uint8  Error            : 1;
        uint8  Interrupt_Enable : 1;
        uint8  Filter_Disable   : 1;
        uint8  Clock_Select     : 2;
      }      bit;
      uint8  reg;
    } FMCSR_reg;  // Control & Status
    //uint8 dummy2[3];
  } Fan_Tach_Monitor[8]; // Fan Tach Monitor

  union {
    struct {
      uint8  Fan_Tach_Monitor_0_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_0_Filter_Select_1 : 1;
      uint8  Fan_Tach_Monitor_1_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_1_Filter_Select_1 : 1;
      uint8  Fan_Tach_Monitor_2_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_2_Filter_Select_1 : 1;
      uint8  Fan_Tach_Monitor_3_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_3_Filter_Select_1 : 1;
    }      bit;
    uint8  reg;
  } FMPFSR0_reg;  // Fan Tach Monitor Programmable Filter Select Register 0-3
  //uint8  dummy6[3];

  union {
    struct {
      uint8  Fan_Tach_Monitor_0_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_0_Filter_Select_1 : 1;
      uint8  Fan_Tach_Monitor_1_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_1_Filter_Select_1 : 1;
      uint8  Fan_Tach_Monitor_2_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_2_Filter_Select_1 : 1;
      uint8  Fan_Tach_Monitor_3_Filter_Select_0 : 1;
      uint8  Fan_Tach_Monitor_3_Filter_Select_1 : 1;
    }      bit;
    uint8  reg;
  } FMPFSR1_reg;  // Fan Tach Monitor Programmable Filter Select Register 4-7
  //uint8  dummy7[3];

} HWREG_PILOT_II_FAN_CONTROL;
#pragma pack ()
